package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_NOT_IN_CONTACT;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.BooleanBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.IntegerBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.LongBoundary;
import java.util.Optional;

public class BiCreaterAppend extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterAppend instance = new BiCreaterAppend();

  private BiCreaterAppend() {
    super();
  }

  public static BiCreaterAppend getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return BoundaryBiCreaterType.APPEND;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1.checkWith(IS_NOT_IN_CONTACT, b2)) {
      if (b1 instanceof BooleanBoundary) {
        return createrBoolean((BooleanBoundary) b1, (BooleanBoundary) b2);
      }
      if (b1 instanceof IntegerBoundary || b1 instanceof LongBoundary) {
        return createrLong(b1, b2);
      }
      if (b1.getUpperBound().equals(b2.getLowerBound())
          && b1.getUpperBoundType() != b2.getLowerBoundType()) {
        return Optional.of(
            b1.getCopy(
                b1.getLowerBound(),
                b2.getUpperBound(),
                b1.getLowerBoundType(),
                b2.getUpperBoundType()));
      } else if (b2.getUpperBound().equals(b1.getLowerBound())
          && b2.getUpperBoundType() != b1.getLowerBoundType()) {
        return Optional.of(
            b1.getCopy(
                b2.getLowerBound(),
                b1.getUpperBound(),
                b2.getLowerBoundType(),
                b1.getUpperBoundType()));
      }
    }
    return Optional.empty();
  }

  private Optional<AbstractGrowingBoundary> createrLong(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    long b1L = (long) b1.getLowerBound();
    long b1U = (long) b1.getUpperBound();
    long b2L = (long) b1.getLowerBound();
    long b2U = (long) b1.getUpperBound();
    if (b1U + 1 == b2L
        && b1.getUpperBoundType() == INCLUSIVE
        && b2.getLowerBoundType() == INCLUSIVE) {
      return Optional.of(
          b1.getCopy(
              b1.getLowerBound(),
              b2.getUpperBound(),
              b1.getLowerBoundType(),
              b2.getUpperBoundType()));
    } else if (b2U + 1 == b1L
        && b2.getUpperBoundType() == INCLUSIVE
        && b1.getLowerBoundType() == INCLUSIVE) {
      return Optional.of(
          b1.getCopy(
              b2.getLowerBound(),
              b1.getUpperBound(),
              b2.getLowerBoundType(),
              b1.getUpperBoundType()));
    }
    return Optional.empty();
  }

  private Optional<AbstractGrowingBoundary> createrBoolean(BooleanBoundary b1, BooleanBoundary b2) {
    if ((b1.getLowerBound() == b1.getUpperBound())
        && (b2.getLowerBound() == b2.getUpperBound())
        && b1.getLowerBound() != b1.getLowerBound()) {
      return Optional.of(new BooleanBoundary(""));
    } else {
      return Optional.empty();
    }
  }
}
