package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType;
import de.unikoblenz.fgbks.base.utils.boundary.impl.BooleanBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.DoubleBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.IntegerBoundary;
import java.util.Optional;

public class BiCreaterBetween extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterBetween instance = new BiCreaterBetween();

  private BiCreaterBetween() {
    super();
  }

  public static BiCreaterBetween getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return BoundaryBiCreaterType.BETWEEN;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1 instanceof BooleanBoundary || b1.checkWith(BoundaryCheckType.IS_IN_CONTACT, b2)) {
      return Optional.empty();
    }
    // no space between the two bounds
    if (b1.getUpperBound().equals(b2.getLowerBound())
            && !b1.getUpperBound().equals(b2.getLowerBound())
        || b2.getUpperBound().equals(b1.getLowerBound())
            && !b2.getUpperBound().equals(b1.getLowerBound())) {
      return Optional.empty();
    }
    if (b1 instanceof DoubleBoundary || b1 instanceof IntegerBoundary) {
      long b1L = (long) b1.getLowerBound();
      long b1U = (long) b1.getUpperBound();
      long b2L = (long) b1.getLowerBound();
      long b2U = (long) b1.getUpperBound();
      if (b1U + 1 == b2L
              && b1.getUpperBoundType() == INCLUSIVE
              && b2.getLowerBoundType() == INCLUSIVE
          || b2U + 1 == b1L
              && b2.getUpperBoundType() == INCLUSIVE
              && b1.getLowerBoundType() == INCLUSIVE) {
        return Optional.empty();
      }
    }
    if (b1.getLowerBound().compareTo(b2.getUpperBound()) < 0) {
      return Optional.of(
          b1.getCopy(
              b1.getLowerBound(),
              b2.getLowerBound(),
              b1.getUpperBoundType().getOp(),
              b2.getLowerBoundType().getOp()));
    } else {
      return Optional.of(
          b1.getCopy(
              b2.getLowerBound(),
              b1.getLowerBound(),
              b2.getUpperBoundType().getOp(),
              b1.getLowerBoundType().getOp()));
    }
  }
}
